#region Copyright Syncfusion Inc. 2001-2017.
// Copyright Syncfusion Inc. 2001-2017. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion
using AutoMapper;
using ScheduleProblem.Models;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;

namespace ScheduleProblem.Controllers
{
    public class HomeController : Controller
    {
        ExampleDataContext db = new ExampleDataContext();

        public ActionResult Index()
        {
            return View();
        }

        public JsonResult GetVersorgung()
        {
            var dienst = db.Versorgungs.Select(x => new VersorgungDto {
                Id = x.VersorgungID,
                LastName = x.LastName,
                FirstName = x.FirstName
            }).ToList();

            return Json(dienst, JsonRequestBehavior.AllowGet);
        }


        public JsonResult GetMitarbeiter(Guid id)
        {
            var mitarbeiterQuery = db.Mitarbeiters.Where(k => k.VersorgungID == id)
                                                       .OrderBy(k => k.LastName)
                                                       .AsEnumerable()
                                                       .Select(s =>
                                                        new MitarbeiterDto
                                                        {
                                                            VersorgungID = s.Versorgung.VersorgungID,
                                                            MitarbeiterID = s.MitarbeiterID,
                                                            MaLastName = s.LastName,
                                                            MaFirstName = s.FirstName,
                                                            MaFullName = $"{s.LastName}, {s.FirstName}",
                                                        }).ToList();

            //var mitarbeiter = Mapper.Map<List<KlientMitarbeiterDto>>(mitarbeiterQuery);

            mitarbeiterQuery.Insert(0, new MitarbeiterDto
            {
                VersorgungID = new Guid("ce445379-4fd9-4d45-a98e-252fd35b5d8a"),
                MitarbeiterID = new Guid("b1050d0a-bc1a-4b6f-8e62-1d7c92ff01c0"),
                MaFullName = "Alle"
            });

            return Json(mitarbeiterQuery, JsonRequestBehavior.AllowGet);
        }

        public JsonResult GetMitarbeiterVersorgung(Guid id)
        {
            var mitarbeiterQuery = db.Mitarbeiters.Where(k => k.VersorgungID == id)
                                                       .OrderBy(k => k.LastName)
                                                       //.AsEnumerable()
                                                       .Select(s =>
                                                        new MitarbeiterDto
                                                        {
                                                            VersorgungID = s.Versorgung.VersorgungID,
                                                            MitarbeiterID = s.MitarbeiterID,
                                                            MaLastName = s.LastName,
                                                            MaFirstName = s.FirstName,
                                                            MaFullName = $"{s.LastName}, {s.FirstName}",
                                                        }).ToList();

            return Json(mitarbeiterQuery, JsonRequestBehavior.AllowGet);
        }


        public JsonResult GetDienstarten(Guid id)
        {
            var dienstartQuery = db.Dienstarts.Where(d => d.VersorgungID == id)
                                        .OrderBy(d => d.Bezeichnung)
                                        .AsEnumerable()
                                        .Select(d => new DienstartDto
                                        {
                                            Abkzg = d.Abkzg,
                                            Bezeichnung = d.Bezeichnung,
                                            DienstartID = d.DienstartID,
                                            VersorgungID = d.VersorgungID,
                                        }).ToList();

            return Json(dienstartQuery, JsonRequestBehavior.AllowGet);
        }




        public JsonResult GetData(Guid id, String CurrentDate, String CurrentView, String CurrentAction, string MitarbeiterIds)
        {

            List<string> Ids = new List<string>();
            if (!String.IsNullOrEmpty(MitarbeiterIds))
            {
                Ids = MitarbeiterIds.Split(',').ToList();
            }

            var dateString = Regex.Match(CurrentDate.ToString(), @"^(\w+\b.*?){4}").ToString();
            string format = "ddd MMM dd yyyy";
            DateTime dateTimeValue;
            try
            {
                dateTimeValue = DateTime.ParseExact(dateString, format, CultureInfo.InvariantCulture);
            }
            catch (FormatException)
            {
                var dateSplit = CurrentDate.Split(' ');//For IE<=10 Fri Mar 20 11:00:22 UTC+0530 2015
                if (dateSplit[2].Length == 1) dateSplit[2] = string.Concat("0", dateSplit[2]);
                dateString = string.Concat(dateSplit[0], ' ', dateSplit[1], ' ', dateSplit[2], ' ', dateSplit[dateSplit.Length - 1]);
                dateTimeValue = DateTime.ParseExact(dateString, format, CultureInfo.InvariantCulture);
            }

            var data = FilterAppointment(id, dateTimeValue, CurrentAction, CurrentView, Ids);
            

            return Json(data, JsonRequestBehavior.AllowGet);
        }



        public List<EventDto> FilterAppointment(Guid id, DateTime CurrentDate, String CurrentAction, String CurrentView, List<string> Ids)
        {
            DateTime CurrDate = Convert.ToDateTime(CurrentDate);
            DateTime StartDate = FirstWeekDate(CurrDate.Date);
            DateTime EndDate = FirstWeekDate(CurrDate.Date);
            List<EventDto> appointmentList = Mapper.Map<List<EventDto>>(new ExampleDataContext().Diensts.ToList());
            switch (CurrentView)
            {
                case "day":
                    StartDate = CurrentDate;
                    EndDate = CurrentDate;
                    break;
                case "week":
                    EndDate = EndDate.AddDays(7);
                    break;
                case "workweek":
                    EndDate = EndDate.AddDays(5);
                    break;
                case "month":
                    StartDate = CurrDate.Date.AddDays(-CurrDate.Day + 1);
                    EndDate = StartDate.AddMonths(1);
                    break;
            }

            if (Ids.Select(Guid.Parse).Contains(new Guid("b1050d0a-bc1a-4b6f-8e62-1d7c92ff01c0")) || Ids.Count == 0)
            {
                appointmentList = Mapper.Map<List<EventDto>>(new ExampleDataContext().Diensts.ToList().Where(app =>
                app.VersorgungID == id &&
                ((Convert.ToDateTime(app.Beginn).Date >= Convert.ToDateTime(StartDate.Date)) &&
                (Convert.ToDateTime(app.Ende).Date <= Convert.ToDateTime(EndDate.Date))))
                .AsEnumerable().Select(d => new EventDto
                {
                    DienstID = d.DienstID,
                    MitarbeiterID = d.MitarbeiterID,
                    VersorgungID = d.VersorgungID,
                    Beginn = d.Beginn,
                    Ende = d.Ende,
                    DienstartAbkzg = d.Dienstart.Abkzg,
                    DienstartText = d.Dienstart.Bezeichnung,
                    MaLastName = d.Mitarbeiter.LastName,
                    MaFirstName = d.Mitarbeiter.FirstName,
                    MaFullName = $"{d.Mitarbeiter.LastName}, {d.Mitarbeiter.FirstName}"
                }).ToList());
            }
            else
            {
                appointmentList = Mapper.Map<List<EventDto>>(new ExampleDataContext().Diensts.ToList().Where(app =>
                app.VersorgungID == id &&
                ((Convert.ToDateTime(app.Beginn).Date >= Convert.ToDateTime(StartDate.Date)) &&
                (Convert.ToDateTime(app.Ende).Date <= Convert.ToDateTime(EndDate.Date))) &&
                (Ids.Select(Guid.Parse).Contains(app.MitarbeiterID))).AsEnumerable().Select(d => new EventDto
                {
                    DienstID = d.DienstID,
                    MitarbeiterID = d.MitarbeiterID,
                    VersorgungID = d.VersorgungID,
                    Beginn = d.Beginn,
                    Ende = d.Ende,
                    DienstartAbkzg = d.Dienstart.Abkzg,
                    DienstartText = d.Dienstart.Bezeichnung,
                    MaLastName = d.Mitarbeiter.LastName,
                    MaFirstName = d.Mitarbeiter.FirstName,
                    MaFullName = $"{d.Mitarbeiter.LastName}, {d.Mitarbeiter.FirstName}"
                }).ToList());
            }


            return appointmentList;
        }

        internal static DateTime FirstWeekDate(DateTime CurrentDate)
        {
            try
            {
                DateTime FirstDayOfWeek = CurrentDate;
                DayOfWeek WeekDay = FirstDayOfWeek.DayOfWeek;
                switch (WeekDay)
                {
                    case DayOfWeek.Sunday:
                        break;
                    case DayOfWeek.Monday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-1);
                        break;
                    case DayOfWeek.Tuesday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-2);
                        break;
                    case DayOfWeek.Wednesday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-3);
                        break;
                    case DayOfWeek.Thursday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-4);
                        break;
                    case DayOfWeek.Friday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-5);
                        break;
                    case DayOfWeek.Saturday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-6);
                        break;
                }
                return (FirstDayOfWeek);
            }
            catch
            {
                return DateTime.Now;
            }
        }

        public ActionResult About()
        {
            ViewBag.Message = "Your application description page.";

            return View();
        }

        public ActionResult Contact()
        {
            ViewBag.Message = "Your contact page.";

            return View();
        }
    }
}